# {{ description }}
{{ name }}(
{%- for name,v in args.items() -%}
{{ "\n\t" }}{{ name }}
    {%- if v.type %}: {{ v.type }}{% endif -%}
    {%- if v.default != 'NOT_SET' %} = {{ v.default }}{%endif-%}
    {%- if not loop.last -%},{%- endif -%}
    {%- if v.comment %} # {{ v.comment }}{% endif -%}
{%- endfor %}
)